from .util import warning
import numpy as np

def message_delay_all(messages):
    """ returns a list of tuples (delay, msg1, msg2) for delays between all messages in the list """
    previous_message=None
    result=[]
    for m in messages:
        if previous_message:
            delay=round(m.timestamp - previous_message.timestamp,6)
            result.append ( (delay , previous_message, m) )
        previous_message=m
    return result
        


def message_delay_same_channel (messages, channel, message_id):
    """ returns a list of tuples (delay, msg1, msg2) for delays between messages having the same id and the same channel """
    relevant_messages=filter(lambda message: message.channel==channel and message.message_id==message_id, messages)
    return message_delay_all(relevant_messages)

def message_delay_cross_channel (messages, source_channel, destination_channel, message_id):
    assert source_channel != destination_channel, "Source and destination channel must be different"
    """ returns a list of tuples (delay, msg1, msg2) for delays between messages appearing first on source channel and then on destination channel"""
    relevant_messages=filter(lambda message: message.channel in (source_channel, destination_channel) and message.message_id==message_id, messages)

    latest_source_message=None
    result=[]
    for m in relevant_messages:
        if m.channel==source_channel:
            if latest_source_message:
                warning("message_delay_cross_channel: two messages on source channel in a row: " + latest_source_message.to_string() + " " + m.to_string())
            latest_source_message=m
        elif m.channel==destination_channel:
            if latest_source_message:
                delay=round(m.timestamp - latest_source_message.timestamp,6)
                result.append ( (delay , latest_source_message, m) )
                latest_source_message=None
            else: # target message without source message, ignore
                pass
        else:
            assert False, "Internal program error, filter did not work as intended"
            
    return result

def calculate_bus_load(messages, channel):
    """ returns a list of tuples (load [from 0..1], msg1, msg2) for the load between ack field of msg1 and ack field of msg2.
        Caution: To determine the average load over time, a simple average of this values is only valid if all intervals are equal. Otherwise, explicit integration
        is required """
    relevant_messages=filter(lambda message: message.channel==channel, messages)

    previous_message=None
    result=[]
    for m in relevant_messages:
        if previous_message:
            interval=round(m.timestamp - previous_message.timestamp,6)
            busy_time=previous_message.calculate_inter_frame_gap()+m.calculate_duration()
            #print delay, busy_time
            result.append ( (busy_time *1.0/ interval , busy_time, interval, previous_message , m) )
        previous_message=m
    return result
    

def filter_channel(messages, channel):
    """ returns a message list containing only messages on the same channel"""
    return filter(lambda message: message.channel==channel, messages)

def sublist(messagelist, start_counter, end_counter):
    """ returns a message list containing only messages from message_list that are between start_number (incl.) and end_number (excl.) """
    return filter(lambda message: message.counter >= start_counter and message.counter < end_counter, messagelist)

    

def stats(values):
    a=np.array(values)

    median=np.median(a) 
    return (len(values),  round(median,6), round(np.mean(a),6), round(np.min(a),6), round(np.max(a),6))
    
    
    
